# This example requires the 'message_content' privileged intent to function.

import asyncio

import discord
import yt_dlp as youtube_dl

import random
import os
from discord.ext import commands, tasks
from pathlib import Path

# Suppress noise about console usage from errors
youtube_dl.utils.bug_reports_message = lambda: ''


ytdl_format_options = {
    'format': 'bestaudio/mp3',
    'outtmpl': '/tmp/discord_%(title)s-%(id)s.%(ext)s',
    'postprocessors': [{
        'key': 'FFmpegExtractAudio',
        'preferredcodec': 'mp3',
        'preferredquality': '192',
        }]
}

ffmpeg_options = {
    'options': '-vn',
}

ytdl = youtube_dl.YoutubeDL(ytdl_format_options)


class YTDLSource(discord.PCMVolumeTransformer):
    def __init__(self, source, *, data, volume=0.5):
        super().__init__(source, volume)

        self.data = data

        self.title = data.get('title')
        self.url = data.get('url')

    @classmethod
    async def from_url(cls, url, *, loop=None, stream=False):
        loop = loop or asyncio.get_event_loop()
        data = await loop.run_in_executor(None, lambda: ytdl.extract_info(url, download=not stream))

        if 'entries' in data:
            # take first item from a playlist
            data = data['entries'][0]

        filename = data['url'] if stream else ytdl.prepare_filename(data)
        extension = filename.split('.')[-1]
        filename = filename.replace(extension, 'mp3')
        return cls(discord.FFmpegPCMAudio(filename, **ffmpeg_options), data=data)

    @classmethod
    async def from_file(cls, audio_file, ctx, *, loop=None, stream=False):
        source = discord.PCMVolumeTransformer(discord.FFmpegPCMAudio(audio_file,**ffmpeg_options))
        ctx.voice_client.play(source, after=lambda e: print('Player error: %s' % e) if e else None)


class Audio(commands.Cog, name="Commandes Audio"):
    def __init__(
            self,
            bot,
            audio_path=None,
            voice_chan=None,
            text_chan=None):
        self.bot = bot
        self.sounds = []
        self.voice_chan = voice_chan
        self.text_chan = text_chan
        self.audio_path = audio_path
        self.sounds_history = []

    @commands.Cog.listener()
    async def on_voice_state_update(self, member, before, after):
        if before.channel is None and after.channel:
            for channel in self.bot.get_all_channels():
                if channel.name == self.text_chan:
                    current_chan = channel
            async for msg in  current_chan.history(limit=1):
                last_message = msg
            ctx = await self.bot.get_context(last_message)
            ctx.message.author = member
            #welcome = bot.get_command('welcome')
            await asyncio.sleep(2)
            await self.welcome(ctx, member.display_name)

    @commands.Cog.listener()
    async def on_ready(self):
        self.play_next.start()
        self.pipelette.start()
        if self.voice_chan:
            await self.join(self.voice_chan)
            #await self.welcome(None, 'kabot')

    @commands.command(help="Toi tu fermes ta gueule! Tu la fermes définitivement")
    async def mute(self, ctx, member: discord.Member=None, mute_time = 10):
        if member.voice.mute:
            async with ctx.channel.typing():
                await asyncio.sleep(1)
            await ctx.channel.send("Tu ne vas pas m'avoir si facilement")
            return
        if not member:
            async with ctx.channel.typing():
                await asyncio.sleep(1)
            await ctx.channel.send("Qui veux-tu mute?")
            return
        if mute_time > 60:
            async with ctx.channel.typing():
                await asyncio.sleep(1)
            await ctx.channel.send("Doucement sur le temps!")
            return
        member = ctx.message.mentions[0]
        await member.edit(mute=True)
        async with ctx.channel.typing():
            await asyncio.sleep(1)
        await ctx.send("Tu sors %s!" % member.mention)
        await asyncio.sleep(mute_time)
        await member.edit(mute=False)
        async with ctx.channel.typing():
            await asyncio.sleep(1)
        await ctx.channel.send("Tu peux reparler %s!" % member.mention)

    @commands.command(help="Appuie sur la detente PUSSY!")
    async def roulette(self, ctx):
        mute_time = 60
        member = ctx.message.author
        if not member.voice:
            async with ctx.typing():
                await ctx.channel.send("You're not in voice channel")
                return
        barillet = [False, False, True, False, False]
        bullet = random.choice(barillet)
        await self.joke(ctx, "spinning_and_closed.mp3")
        if bullet == True:
            await self.joke(ctx, "omawa_shindeiru.mp3")
            await asyncio.sleep(2)
            await member.edit(mute=True)
            async with ctx.typing():
                await asyncio.sleep(3)
            await ctx.channel.send("Perdu, tu es mute pendant 60 secondes!")
        else:
            await self.joke(ctx, "hammer_and_dry.mp3")
            async with ctx.typing():
                await asyncio.sleep(3)
            await ctx.channel.send("Gagné, tu ne seras pas mute!")
            return
        await asyncio.sleep(mute_time)
        await member.edit(mute=False)
        async with ctx.typing():
            await asyncio.sleep(0.5)
        await ctx.channel.send("Tu peux reparler %s!" % member.mention)

    @tasks.loop(seconds=300)
    async def pipelette(self):
        for channel in self.bot.get_all_channels():
            if channel.name == self.voice_chan and\
                    len(channel.members) > 1 and\
                    random.choice([True,True,True]):
                for chan in self.bot.get_all_channels():
                    if chan.name == self.text_chan:
                        current_chan = chan
                        await asyncio.sleep(random.choice(range(305)))
                        async for msg in current_chan.history(limit=1):
                            last_message = msg
                        ctx = await self.bot.get_context(last_message)
                        ctx.message.content = ""
                        await self.joke(ctx)

    @tasks.loop(seconds=1)
    async def play_next(self):
        connected_voice = [chan for chan in self.bot.voice_clients if chan.is_connected()]
        if connected_voice:
            if not [chan for chan in connected_voice if chan.is_playing()]:
                if self.sounds:
                    audio_type, audio_file, ctx = self.sounds[0]
                    if 'url' in audio_type:
                        player = await YTDLSource.from_url(audio_file, loop=self.bot.loop)
                        ctx.voice_client.play(player, after=lambda e: print(f'Player error: {e}') if e else None)
                    elif 'file' in audio_type:
                        player = await YTDLSource.from_file(audio_file, ctx)
                    last_audio = self.sounds.pop(0)
                    self.sounds_history.reverse()
                    self.sounds_history.append(last_audio)
                    self.sounds_history.reverse()
                    if len(self.sounds_history) > 5:
                        self.sounds_history = self.sounds_history[:5]

    @commands.command(help='Troll commands', hidden=True)
    async def okre(self, ctx):
        await self.joke(ctx, "tu_dois_tout_donner.mp3")

    @commands.command(help='Clear sound queue')
    async def clear(self, ctx):
        self.sounds = []

    @commands.command()
    async def welcome(self, ctx, user=None):
        """Joue le son d'accueil de l'utilisateur"""
        if not user:
            user = ctx.message.author.name
        try:
            audio_file = random.choice([f for f in os.listdir(self.audio_path + '%s/' % user) if f.endswith('.mp3')])
            audio_file = self.audio_path + '/%s/' % user + audio_file
        except:
            audio_file = random.choice([f for f in Path(self.audio_path + '/').glob('**/*.mp3')])
        self.sounds.append(('file', audio_file, ctx))

    @commands.command()
    async def repeat(self, ctx):
        """Rejouer le dernier son joué"""
        user = ctx.message.author.name
        source = self.sounds_history[0]
        self.sounds.append(source)

    @commands.command(help="detail du dernier son joué")
    async def last(self, ctx, number = 1):
        number = int(number)
        for sound in self.sounds_history[0:number]:
            await ctx.channel.send("```"+str(sound[1])+"```")

    @commands.command()
    async def join(self, chan_name: discord.VoiceChannel):
        """Joins a voice channel"""

        #if ctx.voice_client is not None:
        #    return await ctx.voice_client.move_to(channel)
        channel = [x for x in self.bot.get_all_channels() if x.name == chan_name][0]
        return await channel.connect()

        await channel.connect()

    @commands.command()
    async def play(self, ctx, *, url):
        """Plays from a url (almost anything youtube_dl supports)"""
        self.sounds.append(('url',url,ctx))

    @commands.command()
    async def stream(self, ctx, *, url):
        """Streams from a url (same as yt, but doesn't predownload)"""

        async with ctx.typing():
            player = await YTDLSource.from_url(url, loop=self.bot.loop, stream=True)
            ctx.voice_client.play(player, after=lambda e: print(f'Player error: {e}') if e else None)

        await ctx.send(f'Now playing: {player.title}')

    @commands.command()
    async def stop(self, ctx):
        """Stoppe la lecture en cours"""
        await ctx.voice_client.stop()

    @commands.command()
    async def volume(self, ctx, volume: int):
        """Changes the player's volume"""

        if ctx.voice_client is None:
            return await ctx.send("Not connected to a voice channel.")

        ctx.voice_client.source.volume = volume / 100
        await ctx.send(f"Changed volume to {volume}%")

    @commands.command()
    async def leave(self, ctx):
        """Stops and disconnects the bot from voice"""

        await ctx.voice_client.disconnect()

    @commands.command()
    async def find(self, ctx, query):
        """Cherche un son dans la bibliothèque"""
        query = query.lower()
        audio_files = [f for f in Path(self.audio_path + '/').glob('**/*.mp3') if query in str(f).lower()]
        if not audio_files:
            await ctx.channel.send("%s not found" % query)
        else:
            for file in audio_files:
                await ctx.channel.send("```"+str(file)+"```")

    @commands.command()
    async def joke(self, ctx, folder=None):
        """Joue un son (aléatoire par défaut)"""
        try:
            user = ctx.message.author.name
            if not folder or not ctx.message.content:
                audio_file = random.choice([f"{f}" for f in Path(self.audio_path + '/').glob('**/*.mp3')])
            elif folder:
                folder = folder.lower()
                audio_files = [f for f in Path(self.audio_path + '/').glob('**/*.mp3') if folder in str(f).lower()]
                if not audio_files:
                    await ctx.channel.send("%s not found" % folder)
                    return
                else:
                    audio_file = random.choice(audio_files)
            self.sounds.append(('file', audio_file, ctx))
        except:
            e = exc_info()[0]
            await ctx.channel.send("Exception when calling joke: %s\n" % e)

    @play.before_invoke
    @joke.before_invoke
    @repeat.before_invoke
    @stream.before_invoke
    async def ensure_voice(self, ctx):
        if ctx.voice_client is None:
            if ctx.author.voice:
                await ctx.author.voice.channel.connect()
            else:
                await ctx.send("You are not connected to a voice channel.")
                raise commands.CommandError("Author not connected to a voice channel.")
        elif ctx.voice_client.is_playing():
            pass


